/*
MERCURY_GEN_STUB_SYNC( open_rpc, open_cb,
        int32_t, HG_FAIL,
        open, (hg_string_t) (int32_t) (uint32_t), ,
        HG_GEN_WITHOUT_BULK, )
*/

/* What one would manually write */

/* Define open_in_t */
typedef struct {
    hg_string_t path;
    int32_t flags;
    uint32_t mode;
} open_in_t;

/* Define hg_proc_open_in_t */
static inline int hg_proc_open_in_t(hg_proc_t proc, void *data)
{
    int ret = HG_SUCCESS;
    open_in_t *struct_data = (open_in_t *) data;

    ret = hg_proc_hg_string_t(proc, &struct_data->path);
    if (ret != HG_SUCCESS) {
        HG_ERROR_DEFAULT("Proc error");
        ret = HG_FAIL;
        return ret;
    }

    ret = hg_proc_int32_t(proc, &struct_data->flags);
    if (ret != HG_SUCCESS) {
        HG_ERROR_DEFAULT("Proc error");
        ret = HG_FAIL;
        return ret;
    }

    ret = hg_proc_uint32_t(proc, &struct_data->mode);
    if (ret != HG_SUCCESS) {
        HG_ERROR_DEFAULT("Proc error");
        ret = HG_FAIL;
        return ret;
    }

    return ret;
}

/* Define open_out_t */
typedef struct {
    int32_t ret;
} open_out_t;

/* Define hg_proc_open_out_t */
static inline int hg_proc_open_out_t(hg_proc_t proc, void *data)
{
    int ret = HG_SUCCESS;
    open_out_t *struct_data = (open_out_t *) data;

    ret = hg_proc_int32_t(proc, &struct_data->ret);
    if (ret != HG_SUCCESS) {
        HG_ERROR_DEFAULT("Proc error");
        ret = HG_FAIL;
        return ret;
    }

    return ret;
}

int open_rpc(const char *pathname, int flags, mode_t mode)
{
    open_in_t  open_in_struct;
    open_out_t open_out_struct;
    hg_request_t request;
    hg_status_t status;
    int hg_ret;
    int open_ret;

    /* Fill input structure */
    open_in_struct.path = pathname;
    open_in_struct.flags = flags;
    open_in_struct.mode = mode;

    /* Forward call to remote addr and get a new request */
    hg_ret = HG_Forward(addr, open_id, &open_in_struct,
            &open_out_struct, &request);
    if (hg_ret != HG_SUCCESS) {
        fprintf(stderr, "Could not forward call\n");
        open_ret = HG_FAIL;
        return open_ret;
    }

    /* Wait for call to be executed and return value to be sent back
 *      * (Request is freed when the call completes)
 *           */
    hg_ret = HG_Wait(request, HG_MAX_IDLE_TIME, &status);
    if (hg_ret != HG_SUCCESS) {
        fprintf(stderr, "Error during wait\n");
        open_ret = HG_FAIL;
        return open_ret;
    }
    if (!status) {
        fprintf(stderr, "Operation did not complete\n");
        open_ret = HG_FAIL;
        return open_ret;
    }

    /* Get output parameters */
    open_ret = open_out_struct.ret;

    return open_ret;
}

int server_posix_open(hg_handle_t handle)
{
    int hg_ret = HG_SUCCESS;

    open_in_t      open_in_struct;
    open_out_t     open_out_struct;

    const char *path;
    int flags;
    mode_t mode;
    int ret;

    /* Get input struct */
    ret = HG_Handler_get_input(handle, &open_in_struct);
    if (ret != HG_SUCCESS) {
        fprintf(stderr, "Could not get input struct\n");
        return ret;
    }

    path = open_in_struct.path;
    flags = open_in_struct.flags;
    mode = open_in_struct.mode;

    /* Call open */
    printf("Calling open with path: %s\n", path);
    ret = open(path, flags, mode);

    /* Fill output structure */
    open_out_struct.ret = ret;

    /* Free handle and send response back */
    ret = HG_Handler_start_output(handle, &open_out_struct);
    if (ret != HG_SUCCESS) {
        fprintf(stderr, "Could not respond\n");
        return ret;
    }

    return hg_ret;
}

/* What the macros generates */

typedef struct \
{   \
    hg_string_t in_param_0; int32_t in_param_1; uint32_t in_param_2;   \
    \
} open_in_t; \
        static inline int hg_proc_open_in_t \
    (hg_proc_t proc, void *data) \
{   \
    int ret = 1; \
    open_in_t *struct_data = (open_in_t *) data; \
    \
    ret = hg_proc_hg_string_t \
            (proc, &struct_data->in_param_0); \
    if (ret != 1) { \
      {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Proc error"); \
}; \
      ret = -1; \
      return ret; \
    } ret = hg_proc_int32_t \
            (proc, &struct_data->in_param_1); \
    if (ret != 1) { \
      {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Proc error"); \
}; \
      ret = -1; \
      return ret; \
    } ret = hg_proc_uint32_t \
            (proc, &struct_data->in_param_2); \
    if (ret != 1) { \
      {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Proc error"); \
}; \
      ret = -1; \
      return ret; \
    }   \
    \
    return ret; \
} \
        typedef struct \
{   \
    int32_t ret;   \
    \
} open_out_t; \
        static inline int hg_proc_open_out_t \
    (hg_proc_t proc, void *data) \
{   \
    int ret = 1; \
    open_out_t *struct_data = (open_out_t *) data; \
    \
    ret = hg_proc_int32_t \
            (proc, &struct_data->ret); \
    if (ret != 1) { \
      {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Proc error"); \
}; \
      ret = -1; \
      return ret; \
    }   \
    \
    return ret; \
} \
        \
        /* Generate client rpc stub */ \
        int32_t \
         \
        open_rpc (hg_string_t in_param_0, int32_t in_param_1, uint32_t in_param_2) \
        { \
            open_in_t in_struct; \
            open_out_t out_struct; \
            int32_t ret; \
            na_class_t *network_class; \
            char *server_name; \
            na_addr_t addr; \
            hg_id_t id; \
             \
            hg_request_t request; \
            hg_status_t status; \
            _Bool hg_initialized; \
             \
            _Bool func_registered; \
            int hg_ret, na_ret; \
            \
            /* Is mercury library initialized */ \
            HG_Initialized(&hg_initialized, &network_class); \
            if (!hg_initialized) { \
                char *na_plugin = getenv("MERCURY_NA_PLUGIN"); \
                if (!na_plugin) na_plugin = "mpi"; \
                if (strcmp("mpi", na_plugin) == 0) { \
                    FILE *config; \
                    network_class = NA_MPI_Init(((void *)0), 0); \
                    /* ***************************************************/ \
                    /* TODO Config crap that we don't want in that macro */ \
                    /* ***************************************************/ \
                    if ((config = fopen("port.cfg", "r")) != ((void *)0)) { \
                        size_t nread; \
                        char mpi_port_name[256]; \
                        nread = fread(mpi_port_name, sizeof(char), 256, config); \
                        if (!nread) {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not read port name"); \
}; \
                        fclose(config); \
                        setenv("MERCURY_PORT_NAME", mpi_port_name, 1); \
                    } \
                } \
                hg_ret = HG_Init(network_class); \
                if (hg_ret != 1) { \
                    {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not initialize function shipper"); \
}; \
                    ret = -1; \
                    goto done; \
                } \
                HG_Atfinalize(hg_finalize_server); \
            } \
             \
            \
            /* Get server_name if set */ \
            server_name = getenv("MERCURY_PORT_NAME"); \
            /* Look up addr id */ \
            na_ret = NA_Addr_lookup(network_class, server_name, &addr); \
            if (na_ret != 1) { \
                {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not lookup addr"); \
}; \
                ret = -1; \
                goto done; \
            } \
            \
            /* Check whether call has already been registered or not */ \
            HG_Registered("open", &func_registered, &id); \
            if (!func_registered) { \
                id = HG_Register("open", hg_proc_open_in_t, \
                hg_proc_open_out_t); \
            } \
            \
             \
            \
            /* Fill input structure */ \
            in_struct.in_param_0 = in_param_0; in_struct.in_param_1 = in_param_1; in_struct.in_param_2 = in_param_2;   \
            \
            /* Forward call to remote addr and get a new request */ \
            hg_ret = HG_Forward(addr, id, &in_struct, &out_struct, &request); \
            if (hg_ret != 1) { \
                {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not forward call"); \
}; \
                ret = -1; \
                goto done; \
            } \
            \
            /* Wait for call to be executed and return value to be sent back
 *              * (Request is freed when the call completes)
 *                           */ \
            hg_ret = HG_Wait(request, (3600*1000), &status); \
            if (hg_ret != 1) { \
                {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Error during wait"); \
}; \
                ret = -1; \
                goto done; \
            } \
            if ( !status ) { \
                {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Operation did not complete"); \
}; \
                ret = -1; \
                goto done; \
            } \
            \
             \
            \
            /* Get output parameters */ \
            ret = out_struct.ret;   \
            \
            done: \
            \
            return ret; \
        } \
        /* Generate server rpc stub */ \
        int \
        open_cb (hg_handle_t handle) \
        { \
                int hg_ret = 1; \
                open_in_t in_struct; \
                open_out_t out_struct; \
                hg_string_t in_param_0; int32_t in_param_1; uint32_t in_param_2;   \
                int32_t ret; \
                void *in_buf, *out_buf; \
                size_t in_buf_size, out_buf_size; \
                hg_proc_t proc; \
                 \
                \
                 \
                \
                /* Get input buffer */ \
                hg_ret = HG_Handler_get_input_buf(handle, &in_buf, &in_buf_size); \
                if (hg_ret != 1) { \
                    {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not get input buffer"); \
}; \
                    goto done; \
                } \
                \
                /* Create a new decoding proc */ \
                hg_proc_create(in_buf, in_buf_size, HG_DECODE, &proc); \
                hg_proc_open_in_t(proc, &in_struct); \
                hg_proc_free(proc); \
                \
                /* Get parameters */ \
                in_param_0 = in_struct.in_param_0; in_param_1 = in_struct.in_param_1; in_param_2 = in_struct.in_param_2;   \
                \
                 \
                 \
                \
                /* Call function */ \
                ret = open (in_param_0, in_param_1, in_param_2); \
                \
                 \
                \
                /* Fill output structure */ \
                out_struct.ret = ret;   \
                \
                /* Create a new encoding proc */ \
                hg_ret = HG_Handler_get_output_buf(handle, &out_buf, &out_buf_size); \
                if (hg_ret != 1) { \
                    {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not get output buffer"); \
}; \
                    goto done; \
                } \
                \
                hg_proc_create(out_buf, out_buf_size, HG_ENCODE, &proc); \
                hg_proc_open_out_t(proc, &out_struct); \
                hg_proc_free(proc); \
                \
                /* Free handle and send response back */ \
                hg_ret = HG_Handler_start_response(handle, ((void *)0), 0); \
                if (hg_ret != 1) { \
                    {              \
  fprintf(stderr, "Error "                \
        "in %s:%d (%s): "                 \
        "%s.\n",                          \
        "/home/jsoumagne/workspace/mercury/source/Testing/test_posix.h", 34, __func__, "Could not respond"); \
}; \
                    goto done; \
                } \
                \
                 \
                \
                /* Also free memory allocated during decoding */ \
                hg_proc_create(((void *)0), 0, HG_FREE, &proc); \
                hg_proc_open_in_t(proc, &in_struct); \
                hg_proc_free(proc); \
                \
                done: \
                \
                return hg_ret; \
            }
